@@ -6,7 +6,7 @@ use Mojo::JSON;
 use Mojo::Util 'deprecated';
 use constant DEBUG => $ENV{MOJO_OPENAPI_DEBUG} || 0;
 
-our $VERSION = '1.16';
+our $VERSION = '1.17';
 my $X_RE = qr{^x-};
 
 has _validator => sub { JSON::Validator::OpenAPI::Mojolicious->new; };
@@ -94,7 +94,7 @@ sub _add_routes {
       }
 
       $endpoint->to(ref $to eq 'ARRAY' ? @$to : $to) if $to;
-      $endpoint->to({'openapi.op_spec' => $op_spec});
+      $endpoint->to({'openapi.op_path' => [$path, $http_method]});
       warn "[OpenAPI] Add route $http_method $path (@{[$endpoint->render]})\n" if DEBUG;
     }
 
@@ -110,10 +110,11 @@ sub _before_render {
   return unless my $self = _self($c);
 
   if ($args->{exception}) {
+    $c->stash(exception => $args->{exception});
     $args->{data} = $self->{renderer}
       ->($c, {errors => [{message => 'Internal server error.', path => '/'}], status => 500});
   }
-  elsif (!$c->stash('openapi.op_spec')) {
+  elsif (!$c->stash('openapi.op_path')) {
     $args->{status} = 404;
     $args->{data}   = $self->{renderer}
       ->($c, {errors => [{message => 'Not found.', path => '/'}], status => 404});
@@ -129,8 +130,16 @@ sub _before_render {
 
 sub _helper_spec {
   my ($c, $path) = @_;
-  return $c->stash('openapi.op_spec') unless defined $path;
-  return $c->stash('openapi.api_spec')->get($path);
+  my ($op_path, $spec);
+
+  for my $r (reverse @{$c->match->stack}) {
+    $spec    ||= $r->{'openapi.api_spec'};
+    $op_path ||= $r->{'openapi.op_path'};
+  }
+
+  return $spec->get($path) if $path;
+  return undef unless $op_path;
+  return $spec->data->{paths}{$op_path->[0]}{$op_path->[1]};
 }
 
 sub _log {
